/*
 * Mapped memory region support.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>

#include <exec/memory_area.h>
#include <systypes/base.h>



/* A mapped region abstraction. */

class MappedRegion : public MemoryArea
{
protected:
  l4_umword_t _flags;
  l4_cap_idx_t _ds;
  address_t _ds_start;

public:
  explicit MappedRegion()
  : MemoryArea(), _flags(0), _ds(L4_INVALID_CAP), _ds_start(0)
  {
  }

  explicit MappedRegion(address_t start, address_t end,
                        l4_umword_t flags, l4_cap_idx_t ds = L4_INVALID_CAP,
                        address_t ds_start = 0)
  : MemoryArea(start, end), _flags(flags), _ds(ds), _ds_start(ds_start)
  {
  }

  virtual MemoryArea *copy()
  { return new MappedRegion(_start, _end, _flags, _ds, _ds_start); }

  /* Access to area properties. */

  virtual l4_umword_t flags()
  { return _flags; }

  virtual l4_cap_idx_t dataspace()
  { return _ds; }

  virtual address_t dataspace_start()
  { return _ds_start; }

  /* Return whether the area provides a mapped region. */

  virtual bool is_mapped()
  { return true; }
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
